@@ -1,6 +1,5 @@
 # encoding: utf-8
 """Tests for IPython.utils.text"""
-from __future__ import print_function
 
 #-----------------------------------------------------------------------------
 #  Copyright (C) 2011  The IPython Development Team
@@ -19,6 +18,7 @@ import random
 import sys
 
 import nose.tools as nt
+from pathlib import Path
 
 from IPython.utils import text
 
@@ -29,44 +29,69 @@ from IPython.utils import text
 def test_columnize():
     """Basic columnize tests."""
     size = 5
-    items = [l*size for l in 'abc']
+    items = [l*size for l in 'abcd']
+
     out = text.columnize(items, displaywidth=80)
-    nt.assert_equal(out, 'aaaaa  bbbbb  ccccc\n')
+    nt.assert_equal(out, 'aaaaa  bbbbb  ccccc  ddddd\n')
+    out = text.columnize(items, displaywidth=25)
+    nt.assert_equal(out, 'aaaaa  ccccc\nbbbbb  ddddd\n')
     out = text.columnize(items, displaywidth=12)
-    nt.assert_equal(out, 'aaaaa  ccccc\nbbbbb\n')
+    nt.assert_equal(out, 'aaaaa  ccccc\nbbbbb  ddddd\n')
     out = text.columnize(items, displaywidth=10)
-    nt.assert_equal(out, 'aaaaa\nbbbbb\nccccc\n')
+    nt.assert_equal(out, 'aaaaa\nbbbbb\nccccc\nddddd\n')
+
+    out = text.columnize(items, row_first=True, displaywidth=80)
+    nt.assert_equal(out, 'aaaaa  bbbbb  ccccc  ddddd\n')
+    out = text.columnize(items, row_first=True, displaywidth=25)
+    nt.assert_equal(out, 'aaaaa  bbbbb\nccccc  ddddd\n')
+    out = text.columnize(items, row_first=True, displaywidth=12)
+    nt.assert_equal(out, 'aaaaa  bbbbb\nccccc  ddddd\n')
+    out = text.columnize(items, row_first=True, displaywidth=10)
+    nt.assert_equal(out, 'aaaaa\nbbbbb\nccccc\nddddd\n')
+
+    out = text.columnize(items, displaywidth=40, spread=True)
+    nt.assert_equal(out, 'aaaaa      bbbbb      ccccc      ddddd\n')
+    out = text.columnize(items, displaywidth=20, spread=True)
+    nt.assert_equal(out, 'aaaaa          ccccc\nbbbbb          ddddd\n')
+    out = text.columnize(items, displaywidth=12, spread=True)
+    nt.assert_equal(out, 'aaaaa  ccccc\nbbbbb  ddddd\n')
+    out = text.columnize(items, displaywidth=10, spread=True)
+    nt.assert_equal(out, 'aaaaa\nbbbbb\nccccc\nddddd\n')
+
 
 def test_columnize_random():
-    """Test with random input to hopfully catch edge case """
-    for nitems in [random.randint(2,70) for i in range(2,20)]:
-        displaywidth = random.randint(20,200)
-        rand_len = [random.randint(2,displaywidth) for i in range(nitems)]
-        items = ['x'*l for l in rand_len]
-        out = text.columnize(items, displaywidth=displaywidth)
-        longer_line = max([len(x) for x in out.split('\n')])
-        longer_element = max(rand_len)
-        if longer_line > displaywidth:
-            print("Columnize displayed something lager than displaywidth : %s " % longer_line)
-            print("longer element : %s " % longer_element)
-            print("displaywidth : %s " % displaywidth)
-            print("number of element : %s " % nitems)
-            print("size of each element :\n %s" % rand_len)
-            assert False
+    """Test with random input to hopefully catch edge case """
+    for row_first in [True, False]:
+        for nitems in [random.randint(2,70) for i in range(2,20)]:
+            displaywidth = random.randint(20,200)
+            rand_len = [random.randint(2,displaywidth) for i in range(nitems)]
+            items = ['x'*l for l in rand_len]
+            out = text.columnize(items, row_first=row_first, displaywidth=displaywidth)
+            longer_line = max([len(x) for x in out.split('\n')])
+            longer_element = max(rand_len)
+            if longer_line > displaywidth:
+                print("Columnize displayed something lager than displaywidth : %s " % longer_line)
+                print("longer element : %s " % longer_element)
+                print("displaywidth : %s " % displaywidth)
+                print("number of element : %s " % nitems)
+                print("size of each element :\n %s" % rand_len)
+                assert False, "row_first={0}".format(row_first)
 
 def test_columnize_medium():
-    """Test with inputs than shouldn't be wider tahn 80 """
+    """Test with inputs than shouldn't be wider than 80"""
     size = 40
     items = [l*size for l in 'abc']
-    out = text.columnize(items, displaywidth=80)
-    nt.assert_equal(out, '\n'.join(items+['']))
+    for row_first in [True, False]:
+        out = text.columnize(items, row_first=row_first, displaywidth=80)
+        nt.assert_equal(out, '\n'.join(items+['']), "row_first={0}".format(row_first))
 
 def test_columnize_long():
     """Test columnize with inputs longer than the display window"""
     size = 11
     items = [l*size for l in 'abc']
-    out = text.columnize(items, displaywidth=size-1)
-    nt.assert_equal(out, '\n'.join(items+['']))
+    for row_first in [True, False]:
+        out = text.columnize(items, row_first=row_first, displaywidth=size-1)
+        nt.assert_equal(out, '\n'.join(items+['']), "row_first={0}".format(row_first))
 
 def eval_formatter_check(f):
     ns = dict(n=12, pi=math.pi, stuff='hello there', os=os, u=u"café", b="café")
@@ -108,13 +133,9 @@ def eval_formatter_no_slicing_check(f):
     
     s = f.format('{stuff[slice(1,4)]}', **ns)
     nt.assert_equal(s, 'ell')
-    
-    if sys.version_info >= (3, 4):
-        # String formatting has changed in Python 3.4, so this now works.
-        s = f.format("{a[:]}", a=[1, 2])
-        nt.assert_equal(s, "[1, 2]")
-    else:
-        nt.assert_raises(SyntaxError, f.format, "{a[:]}")
+
+    s = f.format("{a[:]}", a=[1, 2])
+    nt.assert_equal(s, "[1, 2]")
 
 def test_eval_formatter():
     f = text.EvalFormatter()
@@ -144,19 +165,6 @@ def test_dollar_formatter():
     nt.assert_equal(s, "$HOME")
 
 
-def test_long_substr():
-    data = ['hi']
-    nt.assert_equal(text.long_substr(data), 'hi')
-
-
-def test_long_substr2():
-    data = ['abc', 'abd', 'abf', 'ab']
-    nt.assert_equal(text.long_substr(data), 'ab')
-
-def test_long_substr_empty():
-    data = []
-    nt.assert_equal(text.long_substr(data), '')
-
 def test_strip_email():
     src = """\
         >> >>> def f(x):
@@ -180,6 +188,8 @@ def test_LSString():
     lss = text.LSString("abc\ndef")
     nt.assert_equal(lss.l, ['abc', 'def'])
     nt.assert_equal(lss.s, 'abc def')
+    lss = text.LSString(os.getcwd())
+    nt.assert_is_instance(lss.p[0], Path)
 
 def test_SList():
     sl = text.SList(['a 11', 'b 1', 'a 2'])
@@ -187,4 +197,4 @@ def test_SList():
     nt.assert_equal(sl.s, 'a 11 b 1 a 2')
     nt.assert_equal(sl.grep(lambda x: x.startswith('a')), text.SList(['a 11', 'a 2']))
     nt.assert_equal(sl.fields(0), text.SList(['a', 'b', 'a']))
-    nt.assert_equal(sl.sort(field=1, nums=True), text.SList(['b 1', 'a 2', 'a 11']))
\ No newline at end of file
+    nt.assert_equal(sl.sort(field=1, nums=True), text.SList(['b 1', 'a 2', 'a 11']))
